// Package ships with built-in crate for cli managment of server
// It's possible to create other interfaces with use of other crates inside package
// This is just an example and for fast usages.

use std::{env, process::exit};

use strum::{EnumIter, IntoEnumIterator};

use crate::{decoding::decode_json, networking::{get_json, send_json}};

// Command which we use for basic processing
#[derive(Debug, EnumIter)]
enum KnownArguments {
    Help,
    Send,
    Recieve,
    Wrong,
}

// RESTfull endpoint.
const URL: &str = "https://dixxe.top/cctweaked-beta"; 

// Method to disable program in non-panic way :)
fn gracefull_exit(msg: &str) {
    println!("{msg}");
    exit(0)
}

pub fn basic_interface() {
    let args: Vec<_> = env::args().collect();

    if args.len() < 2 {
        gracefull_exit("Please provide correct arguments or look at --help");
    }
    // Removing execution path probably.
    let args = &args[1..];

    // Translate input args to enum value...
    let arg_type = match args[0].to_lowercase().as_str() {
        "help" => KnownArguments::Help,
        "--help" => KnownArguments::Help,
        "-h" => KnownArguments::Help,
        "send" => KnownArguments::Send,
        "recieve" => KnownArguments::Recieve,
        _ => KnownArguments::Wrong,
    };

    // Match for every enum value specific processing method!
    match arg_type {
        KnownArguments::Help => process_help(args),
        KnownArguments::Send => process_send(args),
        KnownArguments::Recieve => process_recieve(args),
        KnownArguments::Wrong => gracefull_exit("Please provide correct arguments or look at --help"),
    };
}

fn process_help(_: &[String]) {
    println!("All posible arguments:");
    // Printing fancy compile-time checked help message.
    for argument in KnownArguments::iter() {
        match argument {
            KnownArguments::Help => println!("{:?} - Shows help message.", argument),
            KnownArguments::Send => println!("{:?} <content> <msg_type> - sends a cctweaked message to server", argument),
            KnownArguments::Recieve => println!("{:?} - Fetches cctweaked message from server", argument),
            _ => (),
        }
    }
}

fn process_send(args: &[String]) {
    if args.len() < 3 {
        gracefull_exit("Not enough arguments for 'send', please refer to --help");
    }

    // Variables that will be used to assemble decoding::CctweakedMessage
    let input_content = &args[1];
    let input_type = &args[2].parse::<i32>().expect("Wrong message type");

    let resp = send_json(URL.to_owned(), input_content.to_owned(), *input_type)
        .expect("Failed to send json");

    println!("{}, {}", resp.status(), resp.text().unwrap());

}

fn process_recieve(_: &[String]) {

    let resp = get_json(URL.to_owned())
        .expect("Expected a value, got error.");

    println!("{:#?}", decode_json(resp));

}

